# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/network/connection_balance/dlb/v3alpha/dlb.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/network/connection_balance/dlb/v3alpha/dlb.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nIcontrib/envoy/extensions/network/connection_balance/dlb/v3alpha/dlb.proto\x12\x37\x65nvoy.extensions.network.connection_balance.dlb.v3alpha\x1a\x1dudpa/annotations/status.proto\"\xde\x01\n\x03\x44lb\x12\n\n\x02id\x18\x01 \x01(\r\x12\x13\n\x0bmax_retries\x18\x02 \x01(\r\x12\x64\n\x0f\x66\x61llback_policy\x18\x03 \x01(\x0e\x32K.envoy.extensions.network.connection_balance.dlb.v3alpha.Dlb.FallbackPolicy\"P\n\x0e\x46\x61llbackPolicy\x12\x08\n\x04None\x10\x00\x12\x18\n\x14NopConnectionBalance\x10\x01\x12\x1a\n\x16\x45xactConnectionBalance\x10\x02\x42\xc3\x01\nEio.envoyproxy.envoy.extensions.network.connection_balance.dlb.v3alphaB\x08\x44lbProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/network/connection_balance/dlb/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.network.connection_balance.dlb.v3alpha.dlb_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nEio.envoyproxy.envoy.extensions.network.connection_balance.dlb.v3alphaB\010DlbProtoP\001Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/network/connection_balance/dlb/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_DLB']._serialized_start=166
  _globals['_DLB']._serialized_end=388
  _globals['_DLB_FALLBACKPOLICY']._serialized_start=308
  _globals['_DLB_FALLBACKPOLICY']._serialized_end=388
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

BANNED_FILENAMES = [
    "BUILD.gn",
]

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

bad = []
for filename in BANNED_FILENAMES:
    if os.path.exists(filename):
        bad.append(filename)

if bad:
    for file in bad:
        print("%s should not exist" % file)
    sys.exit(1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/filter/http/dynamo/v2/dynamo.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/filter/http/dynamo/v2/dynamo.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/envoy/config/filter/http/dynamo/v2/dynamo.proto\x12\"envoy.config.filter.http.dynamo.v2\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\"\x08\n\x06\x44ynamoB\xcc\x01\n0io.envoyproxy.envoy.config.filter.http.dynamo.v2B\x0b\x44ynamoProtoP\x01ZRgithub.com/envoyproxy/go-control-plane/envoy/config/filter/http/dynamo/v2;dynamov2\xf2\x98\xfe\x8f\x05)\x12\'envoy.extensions.filters.http.dynamo.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.filter.http.dynamo.v2.dynamo_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n0io.envoyproxy.envoy.config.filter.http.dynamo.v2B\013DynamoProtoP\001ZRgithub.com/envoyproxy/go-control-plane/envoy/config/filter/http/dynamo/v2;dynamov2\362\230\376\217\005)\022\'envoy.extensions.filters.http.dynamo.v3\272\200\310\321\006\002\020\001'
  _globals['_DYNAMO']._serialized_start=150
  _globals['_DYNAMO']._serialized_end=158
# @@protoc_insertion_point(module_scope)



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import threading
import unittest

from grpc._cython import cygrpc


def _get_number_active_threads():
    return cygrpc._fork_state.active_thread_count._num_active_threads


@unittest.skipIf(os.name == "nt", "Posix-specific tests")
class ForkPosixTester(unittest.TestCase):
    def setUp(self):
        self._saved_fork_support_flag = cygrpc._GRPC_ENABLE_FORK_SUPPORT
        cygrpc._GRPC_ENABLE_FORK_SUPPORT = True

    def testForkManagedThread(self):
        def cb():
            self.assertEqual(1, _get_number_active_threads())

        thread = cygrpc.ForkManagedThread(cb)
        thread.start()
        thread.join()
        self.assertEqual(0, _get_number_active_threads())

    def testForkManagedThreadThrowsException(self):
        def cb():
            self.assertEqual(1, _get_number_active_threads())
            raise Exception("expected exception")

        thread = cygrpc.ForkManagedThread(cb)
        thread.start()
        thread.join()
        self.assertEqual(0, _get_number_active_threads())

    def tearDown(self):
        cygrpc._GRPC_ENABLE_FORK_SUPPORT = self._saved_fork_support_flag


@unittest.skipUnless(os.name == "nt", "Windows-specific tests")
class ForkWindowsTester(unittest.TestCase):
    def testForkManagedThreadIsNoOp(self):
        def cb():
            pass

        thread = cygrpc.ForkManagedThread(cb)
        thread.start()
        thread.join()


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python2.7

# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import glob
import yaml

os.chdir(os.path.dirname(sys.argv[0]) + "/../..")

out = {}

out["libs"] = [
    {
        "name": "benchmark",
        "build": "private",
        "language": "c++",
        "secure": False,
        "defaults": "benchmark",
        "src": sorted(glob.glob("third_party/benchmark/src/*.cc")),
        "headers": sorted(
            glob.glob("third_party/benchmark/src/*.h")
            + glob.glob("third_party/benchmark/include/benchmark/*.h")
        ),
    }
]

print(yaml.dump(out))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/v3/range.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/v3/range.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x65nvoy/type/v3/range.proto\x12\renvoy.type.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"F\n\nInt64Range\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x03:\x1c\x9a\xc5\x88\x1e\x17\n\x15\x65nvoy.type.Int64Range\"F\n\nInt32Range\x12\r\n\x05start\x18\x01 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x05:\x1c\x9a\xc5\x88\x1e\x17\n\x15\x65nvoy.type.Int32Range\"H\n\x0b\x44oubleRange\x12\r\n\x05start\x18\x01 \x01(\x01\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x01:\x1d\x9a\xc5\x88\x1e\x18\n\x16\x65nvoy.type.DoubleRangeBp\n\x1bio.envoyproxy.envoy.type.v3B\nRangeProtoP\x01Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.v3.range_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033io.envoyproxy.envoy.type.v3B\nRangeProtoP\001Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3\272\200\310\321\006\002\020\002'
  _globals['_INT64RANGE']._loaded_options = None
  _globals['_INT64RANGE']._serialized_options = b'\232\305\210\036\027\n\025envoy.type.Int64Range'
  _globals['_INT32RANGE']._loaded_options = None
  _globals['_INT32RANGE']._serialized_options = b'\232\305\210\036\027\n\025envoy.type.Int32Range'
  _globals['_DOUBLERANGE']._loaded_options = None
  _globals['_DOUBLERANGE']._serialized_options = b'\232\305\210\036\030\n\026envoy.type.DoubleRange'
  _globals['_INT64RANGE']._serialized_start=110
  _globals['_INT64RANGE']._serialized_end=180
  _globals['_INT32RANGE']._serialized_start=182
  _globals['_INT32RANGE']._serialized_end=252
  _globals['_DOUBLERANGE']._serialized_start=254
  _globals['_DOUBLERANGE']._serialized_end=326
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
A module to assist in generating experiment related code and artifacts.
"""

from __future__ import print_function

import collections
from copy import deepcopy
import ctypes
import datetime
import json
import math
import os
import re
import sys

import yaml

_CODEGEN_PLACEHOLDER_TEXT = """
This file contains the autogenerated parts of the experiments API.

It generates two symbols for each experiment.

For the experiment named new_car_project, it generates:

- a function IsNewCarProjectEnabled() that returns true if the experiment
  should be enabled at runtime.

- a macro GRPC_EXPERIMENT_IS_INCLUDED_NEW_CAR_PROJECT that is defined if the
  experiment *could* be enabled at runtime.

The function is used to determine whether to run the experiment or
non-experiment code path.

If the experiment brings significant bloat, the macro can be used to avoid
including the experiment code path in the binary for binaries that are size
sensitive.

By default that includes our iOS and Android builds.

Finally, a small array is included that contains the metadata for each
experiment.

A macro, GRPC_EXPERIMENTS_ARE_FINAL, controls whether we fix experiment
configuration at build time (if it's defined) or allow it to be tuned at
runtime (if it's disabled).

If you are using the Bazel build system, that macro can be configured with
--define=grpc_experiments_are_final=true
"""


def _EXPERIMENTS_TEST_SKELETON(defs, test_body):
    return f"""
#include <grpc/support/port_platform.h>

#include "test/core/experiments/fixtures/experiments.h"

#include <memory>

#include "gtest/gtest.h"

#include "src/core/lib/experiments/config.h"

#ifndef GRPC_EXPERIMENTS_ARE_FINAL
{defs}
TEST(ExperimentsTest, CheckExperimentValuesTest) {{
{test_body}
}}

#endif // GRPC_EXPERIMENTS_ARE_FINAL

int main(int argc, char** argv) {{
  testing::InitGoogleTest(&argc, argv);
  grpc_core::LoadTestOnlyExperimentsFromMetadata(
    grpc_core::g_test_experiment_metadata, grpc_core::kNumTestExperiments);
  return RUN_ALL_TESTS();
}}
"""


def _EXPERIMENTS_EXPECTED_VALUE(name, expected_value):
    return f"""
bool GetExperiment{name}ExpectedValue() {{
{expected_value}
}}
"""


def _EXPERIMENT_CHECK_TEXT(name):
    return f"""
  ASSERT_EQ(grpc_core::Is{name}Enabled(),
            GetExperiment{name}ExpectedValue());
"""


def ToCStr(s, encoding="ascii"):
    if isinstance(s, str):
        s = s.encode(encoding)
    result = ""
    for c in s:
        c = chr(c) if isinstance(c, int) else c
        if not (32 <= ord(c) < 127) or c in ("\\", '"'):
            result += "\\%03o" % ord(c)
        else:
            result += c
    return '"' + result + '"'


def SnakeToPascal(s):
    return "".join(x.capitalize() for x in s.split("_"))


def PutBanner(files, banner, prefix):
    # Print a big comment block into a set of files
    for f in files:
        for line in banner:
            if not line:
                print(prefix, file=f)
            else:
                print("%s %s" % (prefix, line), file=f)
        print(file=f)


def PutCopyright(file, prefix):
    # copy-paste copyright notice from this file
    with open(__file__) as my_source:
        copyright = []
        for line in my_source:
            if line[0] != "#":
                break
        for line in my_source:
            if line[0] == "#":
                copyright.append(line)
                break
        for line in my_source:
            if line[0] != "#":
                break
            copyright.append(line)
        PutBanner([file], [line[2:].rstrip() for line in copyright], prefix)


def AreExperimentsOrdered(experiments):
    # Check that the experiments are ordered by name
    for i in range(1, len(experiments)):
        if experiments[i - 1]["name"] >= experiments[i]["name"]:
            print(
                "Experiments are unordered: %s should be after %s"
                % (experiments[i - 1]["name"], experiments[i]["name"])
            )
            return False
    return True


class ExperimentDefinition:
    def __init__(self, attributes):
        self._error = False
        if "name" not in attributes:
            print("ERROR: experiment with no name: %r" % attributes)
            self._error = True
        if "description" not in attributes:
            print(
                "ERROR: no description for experiment %s" % attributes["name"]
            )
            self._error = True
        if "owner" not in attributes:
            print("ERROR: no owner for experiment %s" % attributes["name"])
            self._error = True
        if "expiry" not in attributes:
            print("ERROR: no expiry for experiment %s" % attributes["name"])
            self._error = True
        if attributes["name"] == "monitoring_experiment":
            if attributes["expiry"] != "never-ever":
                print("ERROR: monitoring_experiment should never expire")
                self._error = True
        if self._error:
            print("Failed to create experiment definition")
            return
        self._allow_in_fuzzing_config = True
        self._uses_polling = False
        self._name = attributes["name"]
        self._description = attributes["description"]
        self._expiry = attributes["expiry"]
        self._default = {}
        self._additional_constraints = {}
        self._test_tags = []
        self._requires = set()

        if "uses_polling" in attributes:
            self._uses_polling = attributes["uses_polling"]

        if "allow_in_fuzzing_config" in attributes:
            self._allow_in_fuzzing_config = attributes[
                "allow_in_fuzzing_config"
            ]

        if "test_tags" in attributes:
            self._test_tags = attributes["test_tags"]

        self._platforms = ["posix"]
        if "platforms" in attributes:
            self._platforms = attributes["platforms"]
            if isinstance(self._platforms, str):
                self._platforms = [self._platforms]

        for requirement in attributes.get("requires", []):
            self._requires.add(requirement)

    def IsValid(self, check_expiry=False):
        if self._error:
            return False
        if (
            self._name == "monitoring_experiment"
            and self._expiry == "never-ever"
        ):
            return True
        expiry = datetime.datetime.strptime(self._expiry, "%Y/%m/%d").date()
        if (
            expiry.month == 11
            or expiry.month == 12
            or (expiry.month == 1 and expiry.day < 15)
        ):
            print(
                "For experiment %s: Experiment expiration is not allowed between Nov 1 and Jan 15 (experiment lists %s)."
                % (self._name, self._expiry)
            )
            self._error = True
            return False
        if not check_expiry:
            return True
        today = datetime.date.today()
        two_quarters_from_now = today + datetime.timedelta(days=180)
        if expiry < today:
            print(
                "WARNING: experiment %s expired on %s"
                % (self._name, self._expiry)
            )
        if expiry > two_quarters_from_now:
            print(
                "WARNING: experiment %s expires far in the future on %s"
                % (self._name, self._expiry)
            )
            print("expiry should be no more than two quarters from now")
        return not self._error

    def AddRolloutSpecification(
        self, allowed_defaults, allowed_platforms, rollout_attributes
    ):
        if self._error:
            return False
        if rollout_attributes["name"] != self._name:
            print(
                "ERROR: Rollout specification does not apply to this"
                " experiment: %s" % self._name
            )
            return False
        for requirement in rollout_attributes.get("requires", []):
            self._requires.add(requirement)
        if "default" not in rollout_attributes:
            print(
                "ERROR: no default for experiment %s"
                % rollout_attributes["name"]
            )
            self._error = True
            return False
        for platform in allowed_platforms:
            if isinstance(rollout_attributes["default"], dict):
                value = rollout_attributes["default"].get(platform, False)
                if isinstance(value, dict):
                    # debug is assumed for all rollouts with additional constraints
                    self._default[platform] = "debug"
                    self._additional_constraints[platform] = value
                    continue
            else:
                value = rollout_attributes["default"]
            if value not in allowed_defaults:
                print(
                    "ERROR: default for experiment %s on platform %s "
                    "is of incorrect format"
                    % (rollout_attributes["name"], platform)
                )
                self._error = True
                return False
            self._default[platform] = value
            self._additional_constraints[platform] = {}
        return True

    @property
    def name(self):
        return self._name

    @property
    def description(self):
        return self._description

    def default(self, platform):
        return self._default.get(platform, False)

    @property
    def test_tags(self):
        return self._test_tags

    @property
    def allow_in_fuzzing_config(self):
        return self._allow_in_fuzzing_config

    @property
    def platforms(self):
        return self._platforms

    def additional_constraints(self, platform):
        return self._additional_constraints.get(platform, {})


class ExperimentsCompiler:
    def __init__(
        self,
        defaults,
        final_return,
        final_define,
        platforms_define,
        bzl_list_for_defaults=None,
    ):
        self._defaults = defaults
        self._final_return = final_return
        self._final_define = final_define
        self._platforms_define = platforms_define
        self._bzl_list_for_defaults = bzl_list_for_defaults
        self._experiment_definitions = collections.OrderedDict()
        self._experiment_rollouts = {}

    def AddExperimentDefinition(self, experiment_definition):
        if experiment_definition.name in self._experiment_definitions:
            print(
                "ERROR: Duplicate experiment definition: %s"
                % experiment_definition.name
            )
            return False
        self._experiment_definitions[experiment_definition.name] = (
            experiment_definition
        )
        return True

    def AddRolloutSpecification(self, rollout_attributes):
        if "name" not in rollout_attributes:
            print(
                "ERROR: experiment with no name: %r in rollout_attribute"
                % rollout_attributes
            )
            return False
        if rollout_attributes["name"] not in self._experiment_definitions:
            print(
                "ERROR: rollout for an undefined experiment: %s ignored"
                % rollout_attributes["name"]
            )
            return False
        return self._experiment_definitions[
            rollout_attributes["name"]
        ].AddRolloutSpecification(
            self._defaults, self._platforms_define, rollout_attributes
        )

    def _FinalizeExperiments(self):
        queue = collections.OrderedDict()
        for name, exp in self._experiment_definitions.items():
            queue[name] = exp._requires
        done = set()
        final = collections.OrderedDict()
        while queue:
            take = None
            for name, requires in queue.items():
                if requires.issubset(done):
                    take = name
                    break
            if take is None:
                print("ERROR: circular dependency in experiments")
                return False
            done.add(take)
            final[take] = self._experiment_definitions[take]
            del queue[take]
        self._experiment_definitions = final
        return True

    def _GenerateExperimentsHdrForPlatform(self, platform, file_desc):
        for _, exp in self._experiment_definitions.items():
            define_fmt = self._final_define[exp.default(platform)]
            if define_fmt:
                print(
                    define_fmt
                    % ("GRPC_EXPERIMENT_IS_INCLUDED_%s" % exp.name.upper()),
                    file=file_desc,
                )
            print(
                "inline bool Is%sEnabled() { %s }"
                % (
                    SnakeToPascal(exp.name),
                    self._final_return[exp.default(platform)],
                ),
                file=file_desc,
            )

    def GenerateExperimentsHdr(self, output_file, mode):
        assert self._FinalizeExperiments()
        with open(output_file, "w") as H:
            PutCopyright(H, "//")
            PutBanner(
                [H],
                ["Auto generated by tools/codegen/core/gen_experiments.py"]
                + _CODEGEN_PLACEHOLDER_TEXT.splitlines(),
                "//",
            )

            if mode != "test":
                include_guard = "GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H"
            else:
                real_output_file = output_file.replace(".github", "")
                file_path_list = real_output_file.split("/")[0:-1]
                file_name = real_output_file.split("/")[-1].split(".")[0]

                include_guard = f"GRPC_{'_'.join(path.upper() for path in file_path_list)}_{file_name.upper()}_H"

            print(f"#ifndef {include_guard}", file=H)
            print(f"#define {include_guard}", file=H)
            print(file=H)
            print("#include <grpc/support/port_platform.h>", file=H)
            print(file=H)
            print('#include "src/core/lib/experiments/config.h"', file=H)
            print(file=H)
            print("namespace grpc_core {", file=H)
            print(file=H)
            print("#ifdef GRPC_EXPERIMENTS_ARE_FINAL", file=H)
            idx = 0
            for platform in sorted(self._platforms_define.keys()):
                if platform == "posix":
                    continue
                print(
                    f"\n#{'if' if idx ==0 else 'elif'} "
                    f"defined({self._platforms_define[platform]})",
                    file=H,
                )
                self._GenerateExperimentsHdrForPlatform(platform, H)
                idx += 1
            print("\n#else", file=H)
            self._GenerateExperimentsHdrForPlatform("posix", H)
            print("#endif", file=H)
            print("\n#else", file=H)
            if mode == "test":
                num_experiments_var_name = "kNumTestExperiments"
                experiments_metadata_var_name = "g_test_experiment_metadata"
            else:
                num_experiments_var_name = "kNumExperiments"
                experiments_metadata_var_name = "g_experiment_metadata"
            print("enum ExperimentIds {", file=H)
            for exp in self._experiment_definitions.values():
                print(f"  kExperimentId{SnakeToPascal(exp.name)},", file=H)
            print(f"  {num_experiments_var_name}", file=H)
            print("};", file=H)
            for exp in self._experiment_definitions.values():
                print(
                    "#define GRPC_EXPERIMENT_IS_INCLUDED_%s" % exp.name.upper(),
                    file=H,
                )
                print(
                    "inline bool Is%sEnabled() { return"
                    " Is%sExperimentEnabled<kExperimentId%s>(); }"
                    % (
                        SnakeToPascal(exp.name),
                        "Test" if mode == "test" else "",
                        SnakeToPascal(exp.name),
                    ),
                    file=H,
                )
            print(file=H)
            print(
                (
                    "extern const ExperimentMetadata"
                    f" {experiments_metadata_var_name}[{num_experiments_var_name}];"
                ),
                file=H,
            )
            print(file=H)
            print("#endif", file=H)
            print("}  // namespace grpc_core", file=H)
            print(file=H)
            print(f"#endif  // {include_guard}", file=H)

    def _GenerateExperimentsSrcForPlatform(self, platform, mode, file_desc):
        print("namespace {", file=file_desc)
        have_defaults = set()
        for _, exp in self._experiment_definitions.items():
            print(
                "const char* const description_%s = %s;"
                % (exp.name, ToCStr(exp.description)),
                file=file_desc,
            )
            print(
                "const char* const additional_constraints_%s = %s;"
                % (
                    exp.name,
                    ToCStr(json.dumps(exp.additional_constraints(platform))),
                ),
                file=file_desc,
            )
            have_defaults.add(self._defaults[exp.default(platform)])
            if exp._requires:
                print(
                    "const uint8_t required_experiments_%s[] = {%s};"
                    % (
                        exp.name,
                        ",".join(
                            f"static_cast<uint8_t>(grpc_core::kExperimentId{SnakeToPascal(name)})"
                            for name in sorted(exp._requires)
                        ),
                    ),
                    file=file_desc,
                )
        if "kDefaultForDebugOnly" in have_defaults:
            print("#ifdef NDEBUG", file=file_desc)
            if "kDefaultForDebugOnly" in have_defaults:
                print(
                    "const bool kDefaultForDebugOnly = false;", file=file_desc
                )
            print("#else", file=file_desc)
            if "kDefaultForDebugOnly" in have_defaults:
                print("const bool kDefaultForDebugOnly = true;", file=file_desc)
            print("#endif", file=file_desc)
        print("}  // namespace", file=file_desc)
        print(file=file_desc)
        print("namespace grpc_core {", file=file_desc)
        print(file=file_desc)
        if mode == "test":
            experiments_metadata_var_name = "g_test_experiment_metadata"
        else:
            experiments_metadata_var_name = "g_experiment_metadata"
        print(
            f"const ExperimentMetadata {experiments_metadata_var_name}[] = {{",
            file=file_desc,
        )
        for _, exp in self._experiment_definitions.items():
            print(
                "  {%s, description_%s, additional_constraints_%s, %s, %d, %s, %s},"
                % (
                    ToCStr(exp.name),
                    exp.name,
                    exp.name,
                    (
                        f"required_experiments_{exp.name}"
                        if exp._requires
                        else "nullptr"
                    ),
                    len(exp._requires),
                    self._defaults[exp.default(platform)],
                    "true" if exp.allow_in_fuzzing_config else "false",
                ),
                file=file_desc,
            )
        print("};", file=file_desc)
        print(file=file_desc)
        print("}  // namespace grpc_core", file=file_desc)

    def GenerateExperimentsSrc(self, output_file, header_file_path, mode):
        assert self._FinalizeExperiments()
        with open(output_file, "w") as C:
            PutCopyright(C, "//")
            PutBanner(
                [C],
                ["Auto generated by tools/codegen/core/gen_experiments.py"],
                "//",
            )

            any_requires = False
            for _, exp in self._experiment_definitions.items():
                if exp._requires:
                    any_requires = True
                    break

            print("#include <grpc/support/port_platform.h>", file=C)
            print(file=C)
            if any_requires:
                print("#include <stdint.h>", file=C)
                print(file=C)
            print(
                f'#include "{header_file_path.replace(".github", "")}"', file=C
            )
            print(file=C)
            print("#ifndef GRPC_EXPERIMENTS_ARE_FINAL", file=C)
            idx = 0
            for platform in sorted(self._platforms_define.keys()):
                if platform == "posix":
                    continue
                print(
                    f"\n#{'if' if idx ==0 else 'elif'} "
                    f"defined({self._platforms_define[platform]})",
                    file=C,
                )
                self._GenerateExperimentsSrcForPlatform(platform, mode, C)
                idx += 1
            print("\n#else", file=C)
            self._GenerateExperimentsSrcForPlatform("posix", mode, C)
            print("#endif", file=C)
            print("#endif", file=C)

    def _GenTestExperimentsExpectedValues(self, platform):
        defs = ""
        for _, exp in self._experiment_definitions.items():
            defs += _EXPERIMENTS_EXPECTED_VALUE(
                SnakeToPascal(exp.name),
                self._final_return[exp.default(platform)],
            )
        return defs

    def GenTest(self, output_file):
        assert self._FinalizeExperiments()
        with open(output_file, "w") as C:
            PutCopyright(C, "//")
            PutBanner(
                [C],
                ["Auto generated by tools/codegen/core/gen_experiments.py"],
                "//",
            )
            defs = ""
            test_body = ""
            idx = 0
            for platform in sorted(self._platforms_define.keys()):
                if platform == "posix":
                    continue
                defs += (
                    f"\n#{'if' if idx ==0 else 'elif'} "
                    f"defined({self._platforms_define[platform]})"
                )
                defs += self._GenTestExperimentsExpectedValues(platform)
                idx += 1
            defs += "\n#else"
            defs += self._GenTestExperimentsExpectedValues("posix")
            defs += "#endif\n"
            for _, exp in self._experiment_definitions.items():
                test_body += _EXPERIMENT_CHECK_TEXT(SnakeToPascal(exp.name))
            print(_EXPERIMENTS_TEST_SKELETON(defs, test_body), file=C)

    def _ExperimentEnableSet(self, name):
        s = set()
        s.add(name)
        for exp in self._experiment_definitions[name]._requires:
            for req in self._ExperimentEnableSet(exp):
                s.add(req)
        return s

    def EnsureNoDebugExperiments(self):
        for name, exp in self._experiment_definitions.items():
            for platform, default in exp._default.items():
                if default == "debug":
                    raise ValueError(
                        f"Debug experiments are prohibited. '{name}' is configured with {exp._default}"
                    )

    def GenExperimentsBzl(self, mode, output_file):
        assert self._FinalizeExperiments()
        if self._bzl_list_for_defaults is None:
            return

        defaults = dict(
            (key, collections.defaultdict(list))
            for key in self._bzl_list_for_defaults.keys()
            if key is not None
        )

        bzl_to_tags_to_experiments = dict(
            (platform, deepcopy(defaults))
            for platform in self._platforms_define.keys()
        )

        for platform in self._platforms_define.keys():
            for _, exp in self._experiment_definitions.items():
                if "all" not in exp.platforms and platform not in exp.platforms:
                    continue
                for tag in exp.test_tags:
                    # Search through default values for all platforms.
                    default = exp.default(platform)
                    # Interpret the debug default value as True to switch the
                    # experiment to the "on" mode.
                    if default == "debug":
                        default = True
                    bzl_to_tags_to_experiments[platform][default][tag].append(
                        exp.name
                    )

        with open(output_file, "w") as B:
            PutCopyright(B, "#")
            PutBanner(
                [B],
                ["Auto generated by tools/codegen/core/gen_experiments.py"],
                "#",
            )

            print(
                (
                    '"""Dictionary of tags to experiments so we know when to'
                    ' test different experiments."""'
                ),
                file=B,
            )

            print(file=B)
            if mode == "test":
                print("TEST_EXPERIMENT_ENABLES = {", file=B)
            else:
                print("EXPERIMENT_ENABLES = {", file=B)
            for name, exp in self._experiment_definitions.items():
                print(
                    f"    \"{name}\": \"{','.join(sorted(self._ExperimentEnableSet(name)))}\",",
                    file=B,
                )
            print("}", file=B)

            # Generate a list of experiments that use polling.
            print(file=B)
            if mode == "test":
                print("TEST_EXPERIMENT_POLLERS = [", file=B)
            else:
                print("EXPERIMENT_POLLERS = [", file=B)
            for name, exp in self._experiment_definitions.items():
                if exp._uses_polling:
                    print(f'    "{name}",', file=B)
            print("]", file=B)

            print(file=B)
            if mode == "test":
                print("TEST_EXPERIMENTS = {", file=B)
            else:
                print("EXPERIMENTS = {", file=B)

            for platform in self._platforms_define.keys():
                bzl_to_tags_to_experiments_platform = sorted(
                    (self._bzl_list_for_defaults[default], tags_to_experiments)
                    for default, tags_to_experiments in bzl_to_tags_to_experiments[
                        platform
                    ].items()
                    if self._bzl_list_for_defaults[default] is not None
                )
                print('    "%s": {' % platform, file=B)
                for (
                    key,
                    tags_to_experiments,
                ) in bzl_to_tags_to_experiments_platform:
                    print('        "%s": {' % key, file=B)
                    for tag, experiments in sorted(tags_to_experiments.items()):
                        print('            "%s": [' % tag, file=B)
                        for experiment in sorted(experiments):
                            print('                "%s",' % experiment, file=B)
                        print("            ],", file=B)
                    print("        },", file=B)
                print("    },", file=B)
            print("}", file=B)



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""gRPC Python helloworld.Greeter client with health checking."""

import logging
from time import sleep

import grpc
from grpc_health.v1 import health_pb2
from grpc_health.v1 import health_pb2_grpc
import helloworld_pb2
import helloworld_pb2_grpc


def unary_call(stub: helloworld_pb2_grpc.GreeterStub, message: str):
    response = stub.SayHello(
        helloworld_pb2.HelloRequest(name=message), timeout=3
    )
    print(f"Greeter client received: {response.message}")


def health_check_call(stub: health_pb2_grpc.HealthStub):
    request = health_pb2.HealthCheckRequest(service="helloworld.Greeter")
    resp = stub.Check(request)
    if resp.status == health_pb2.HealthCheckResponse.SERVING:
        print("server is serving")
    elif resp.status == health_pb2.HealthCheckResponse.NOT_SERVING:
        print("server stopped serving")


def run():
    with grpc.insecure_channel("localhost:50051") as channel:
        stub = helloworld_pb2_grpc.GreeterStub(channel)
        health_stub = health_pb2_grpc.HealthStub(channel)
        # Should succeed
        unary_call(stub, "you")

        # Check health status every 1 second for 30 seconds
        for _ in range(30):
            health_check_call(health_stub)
            sleep(1)


if __name__ == "__main__":
    logging.basicConfig()
    run()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/core/v3/base.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/core/v3/base.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import address_pb2 as envoy_dot_config_dot_core_dot_v3_dot_address__pb2
from envoy.config.core.v3 import backoff_pb2 as envoy_dot_config_dot_core_dot_v3_dot_backoff__pb2
from envoy.config.core.v3 import http_uri_pb2 as envoy_dot_config_dot_core_dot_v3_dot_http__uri__pb2
from envoy.type.v3 import percent_pb2 as envoy_dot_type_dot_v3_dot_percent__pb2
from envoy.type.v3 import semantic_version_pb2 as envoy_dot_type_dot_v3_dot_semantic__version__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.core.v3 import context_params_pb2 as xds_dot_core_dot_v3_dot_context__params__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x65nvoy/config/core/v3/base.proto\x12\x14\x65nvoy.config.core.v3\x1a\"envoy/config/core/v3/address.proto\x1a\"envoy/config/core/v3/backoff.proto\x1a#envoy/config/core/v3/http_uri.proto\x1a\x1b\x65nvoy/type/v3/percent.proto\x1a$envoy/type/v3/semantic_version.proto\x1a\x19google/protobuf/any.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a xds/core/v3/context_params.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"]\n\x08Locality\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x0c\n\x04zone\x18\x02 \x01(\t\x12\x10\n\x08sub_zone\x18\x03 \x01(\t:!\x9a\xc5\x88\x1e\x1c\n\x1a\x65nvoy.api.v2.core.Locality\"\x91\x01\n\x0c\x42uildVersion\x12/\n\x07version\x18\x01 \x01(\x0b\x32\x1e.envoy.type.v3.SemanticVersion\x12)\n\x08metadata\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct:%\x9a\xc5\x88\x1e \n\x1e\x65nvoy.api.v2.core.BuildVersion\"\xcf\x01\n\tExtension\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12$\n\x0ftype_descriptor\x18\x03 \x01(\tB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x33\n\x07version\x18\x04 \x01(\x0b\x32\".envoy.config.core.v3.BuildVersion\x12\x10\n\x08\x64isabled\x18\x05 \x01(\x08\x12\x11\n\ttype_urls\x18\x06 \x03(\t:\"\x9a\xc5\x88\x1e\x1d\n\x1b\x65nvoy.api.v2.core.Extension\"\x8a\x05\n\x04Node\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07\x63luster\x18\x02 \x01(\t\x12)\n\x08metadata\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12M\n\x12\x64ynamic_parameters\x18\x0c \x03(\x0b\x32\x31.envoy.config.core.v3.Node.DynamicParametersEntry\x12\x30\n\x08locality\x18\x04 \x01(\x0b\x32\x1e.envoy.config.core.v3.Locality\x12\x17\n\x0fuser_agent_name\x18\x06 \x01(\t\x12\x1c\n\x12user_agent_version\x18\x07 \x01(\tH\x00\x12\x46\n\x18user_agent_build_version\x18\x08 \x01(\x0b\x32\".envoy.config.core.v3.BuildVersionH\x00\x12\x33\n\nextensions\x18\t \x03(\x0b\x32\x1f.envoy.config.core.v3.Extension\x12\x17\n\x0f\x63lient_features\x18\n \x03(\t\x12G\n\x13listening_addresses\x18\x0b \x03(\x0b\x32\x1d.envoy.config.core.v3.AddressB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x1aT\n\x16\x44ynamicParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.xds.core.v3.ContextParams:\x02\x38\x01:\x1d\x9a\xc5\x88\x1e\x18\n\x16\x65nvoy.api.v2.core.NodeB\x19\n\x17user_agent_version_typeJ\x04\x08\x05\x10\x06R\rbuild_version\"\x90\x03\n\x08Metadata\x12Y\n\x0f\x66ilter_metadata\x18\x01 \x03(\x0b\x32\x32.envoy.config.core.v3.Metadata.FilterMetadataEntryB\x0c\xfa\x42\t\x9a\x01\x06\"\x04r\x02\x10\x01\x12\x64\n\x15typed_filter_metadata\x18\x02 \x03(\x0b\x32\x37.envoy.config.core.v3.Metadata.TypedFilterMetadataEntryB\x0c\xfa\x42\t\x9a\x01\x06\"\x04r\x02\x10\x01\x1aN\n\x13\x46ilterMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct:\x02\x38\x01\x1aP\n\x18TypedFilterMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:\x02\x38\x01:!\x9a\xc5\x88\x1e\x1c\n\x1a\x65nvoy.api.v2.core.Metadata\"c\n\rRuntimeUInt32\x12\x15\n\rdefault_value\x18\x02 \x01(\r\x12\x13\n\x0bruntime_key\x18\x03 \x01(\t:&\x9a\xc5\x88\x1e!\n\x1f\x65nvoy.api.v2.core.RuntimeUInt32\"T\n\x0eRuntimePercent\x12-\n\rdefault_value\x18\x01 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12\x13\n\x0bruntime_key\x18\x02 \x01(\t\"c\n\rRuntimeDouble\x12\x15\n\rdefault_value\x18\x01 \x01(\x01\x12\x13\n\x0bruntime_key\x18\x02 \x01(\t:&\x9a\xc5\x88\x1e!\n\x1f\x65nvoy.api.v2.core.RuntimeDouble\"\x93\x01\n\x12RuntimeFeatureFlag\x12;\n\rdefault_value\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValueB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x13\n\x0bruntime_key\x18\x02 \x01(\t:+\x9a\xc5\x88\x1e&\n$envoy.api.v2.core.RuntimeFeatureFlag\"K\n\x08KeyValue\x12#\n\x03key\x18\x01 \x01(\tB\x16\x18\x01\xfa\x42\x08r\x06\x10\x01(\x80\x80\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x1a\n\x05value\x18\x02 \x01(\x0c\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\"O\n\x0cKeyValuePair\x12\x18\n\x03key\x18\x01 \x01(\tB\x0b\xfa\x42\x08r\x06\x10\x01(\x80\x80\x01\x12%\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.Value\"\xde\x02\n\x0eKeyValueAppend\x12\x32\n\x06record\x18\x03 \x01(\x0b\x32\".envoy.config.core.v3.KeyValuePair\x12\x42\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x1e.envoy.config.core.v3.KeyValueB\x13\x18\x01\xfa\x42\x05\x8a\x01\x02\x08\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12S\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32\x39.envoy.config.core.v3.KeyValueAppend.KeyValueAppendActionB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\"\x7f\n\x14KeyValueAppendAction\x12\x1b\n\x17\x41PPEND_IF_EXISTS_OR_ADD\x10\x00\x12\x11\n\rADD_IF_ABSENT\x10\x01\x12\x1e\n\x1aOVERWRITE_IF_EXISTS_OR_ADD\x10\x02\x12\x17\n\x13OVERWRITE_IF_EXISTS\x10\x03\"c\n\x10KeyValueMutation\x12\x34\n\x06\x61ppend\x18\x01 \x01(\x0b\x32$.envoy.config.core.v3.KeyValueAppend\x12\x19\n\x06remove\x18\x02 \x01(\tB\t\xfa\x42\x06r\x04(\x80\x80\x01\"5\n\x0eQueryParameter\x12\x14\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\r\n\x05value\x18\x02 \x01(\t\"\xb7\x01\n\x0bHeaderValue\x12\x1e\n\x03key\x18\x01 \x01(\tB\x11\xfa\x42\x0er\x0c\x10\x01(\x80\x80\x01\xc0\x01\x01\xc8\x01\x00\x12\x30\n\x05value\x18\x02 \x01(\tB!\xfa\x42\x0cr\n(\x80\x80\x01\xc0\x01\x02\xc8\x01\x00\xf2\x98\xfe\x8f\x05\x0c\x12\nvalue_type\x12\x30\n\traw_value\x18\x03 \x01(\x0c\x42\x1d\xfa\x42\x08z\x06\x10\x00\x18\x80\x80\x01\xf2\x98\xfe\x8f\x05\x0c\x12\nvalue_type:$\x9a\xc5\x88\x1e\x1f\n\x1d\x65nvoy.api.v2.core.HeaderValue\"\xab\x03\n\x11HeaderValueOption\x12;\n\x06header\x18\x01 \x01(\x0b\x32!.envoy.config.core.v3.HeaderValueB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x37\n\x06\x61ppend\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.BoolValueB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12[\n\rappend_action\x18\x03 \x01(\x0e\x32:.envoy.config.core.v3.HeaderValueOption.HeaderAppendActionB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x18\n\x10keep_empty_value\x18\x04 \x01(\x08\"}\n\x12HeaderAppendAction\x12\x1b\n\x17\x41PPEND_IF_EXISTS_OR_ADD\x10\x00\x12\x11\n\rADD_IF_ABSENT\x10\x01\x12\x1e\n\x1aOVERWRITE_IF_EXISTS_OR_ADD\x10\x02\x12\x17\n\x13OVERWRITE_IF_EXISTS\x10\x03:*\x9a\xc5\x88\x1e%\n#envoy.api.v2.core.HeaderValueOption\"c\n\tHeaderMap\x12\x32\n\x07headers\x18\x01 \x03(\x0b\x32!.envoy.config.core.v3.HeaderValue:\"\x9a\xc5\x88\x1e\x1d\n\x1b\x65nvoy.api.v2.core.HeaderMap\")\n\x10WatchedDirectory\x12\x15\n\x04path\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"\xfd\x01\n\nDataSource\x12\x1b\n\x08\x66ilename\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x12\x16\n\x0cinline_bytes\x18\x02 \x01(\x0cH\x00\x12\x17\n\rinline_string\x18\x03 \x01(\tH\x00\x12\'\n\x14\x65nvironment_variable\x18\x04 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x12\x41\n\x11watched_directory\x18\x05 \x01(\x0b\x32&.envoy.config.core.v3.WatchedDirectory:#\x9a\xc5\x88\x1e\x1e\n\x1c\x65nvoy.api.v2.core.DataSourceB\x10\n\tspecifier\x12\x03\xf8\x42\x01\"\xe3\x04\n\x0bRetryPolicy\x12=\n\x0eretry_back_off\x18\x01 \x01(\x0b\x32%.envoy.config.core.v3.BackoffStrategy\x12\x46\n\x0bnum_retries\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x13\xf2\x98\xfe\x8f\x05\r\n\x0bmax_retries\x12\x10\n\x08retry_on\x18\x03 \x01(\t\x12G\n\x0eretry_priority\x18\x04 \x01(\x0b\x32/.envoy.config.core.v3.RetryPolicy.RetryPriority\x12R\n\x14retry_host_predicate\x18\x05 \x03(\x0b\x32\x34.envoy.config.core.v3.RetryPolicy.RetryHostPredicate\x12)\n!host_selection_retry_max_attempts\x18\x06 \x01(\x03\x1a\x63\n\rRetryPriority\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12,\n\x0ctyped_config\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x42\r\n\x0b\x63onfig_type\x1ah\n\x12RetryHostPredicate\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12,\n\x0ctyped_config\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x42\r\n\x0b\x63onfig_type:$\x9a\xc5\x88\x1e\x1f\n\x1d\x65nvoy.api.v2.core.RetryPolicy\"\xca\x01\n\x10RemoteDataSource\x12\x39\n\x08http_uri\x18\x01 \x01(\x0b\x32\x1d.envoy.config.core.v3.HttpUriB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x17\n\x06sha256\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x37\n\x0cretry_policy\x18\x03 \x01(\x0b\x32!.envoy.config.core.v3.RetryPolicy:)\x9a\xc5\x88\x1e$\n\"envoy.api.v2.core.RemoteDataSource\"\xba\x01\n\x0f\x41syncDataSource\x12\x31\n\x05local\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceH\x00\x12\x38\n\x06remote\x18\x02 \x01(\x0b\x32&.envoy.config.core.v3.RemoteDataSourceH\x00:(\x9a\xc5\x88\x1e#\n!envoy.api.v2.core.AsyncDataSourceB\x10\n\tspecifier\x12\x03\xf8\x42\x01\"\x9d\x01\n\x0fTransportSocket\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12,\n\x0ctyped_config\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00:(\x9a\xc5\x88\x1e#\n!envoy.api.v2.core.TransportSocketB\r\n\x0b\x63onfig_typeJ\x04\x08\x02\x10\x03R\x06\x63onfig\"\xa5\x01\n\x18RuntimeFractionalPercent\x12\x41\n\rdefault_value\x18\x01 \x01(\x0b\x32 .envoy.type.v3.FractionalPercentB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x13\n\x0bruntime_key\x18\x02 \x01(\t:1\x9a\xc5\x88\x1e,\n*envoy.api.v2.core.RuntimeFractionalPercent\"I\n\x0c\x43ontrolPlane\x12\x12\n\nidentifier\x18\x01 \x01(\t:%\x9a\xc5\x88\x1e \n\x1e\x65nvoy.api.v2.core.ControlPlane*(\n\x0fRoutingPriority\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x08\n\x04HIGH\x10\x01*\x89\x01\n\rRequestMethod\x12\x16\n\x12METHOD_UNSPECIFIED\x10\x00\x12\x07\n\x03GET\x10\x01\x12\x08\n\x04HEAD\x10\x02\x12\x08\n\x04POST\x10\x03\x12\x07\n\x03PUT\x10\x04\x12\n\n\x06\x44\x45LETE\x10\x05\x12\x0b\n\x07\x43ONNECT\x10\x06\x12\x0b\n\x07OPTIONS\x10\x07\x12\t\n\x05TRACE\x10\x08\x12\t\n\x05PATCH\x10\t*>\n\x10TrafficDirection\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0b\n\x07INBOUND\x10\x01\x12\x0c\n\x08OUTBOUND\x10\x02\x42}\n\"io.envoyproxy.envoy.config.core.v3B\tBaseProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.core.v3.base_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\"io.envoyproxy.envoy.config.core.v3B\tBaseProtoP\001ZBgithub.com/envoyproxy/go-control-plane/envoy/config/core/v3;corev3\272\200\310\321\006\002\020\002'
  _globals['_LOCALITY']._loaded_options = None
  _globals['_LOCALITY']._serialized_options = b'\232\305\210\036\034\n\032envoy.api.v2.core.Locality'
  _globals['_BUILDVERSION']._loaded_options = None
  _globals['_BUILDVERSION']._serialized_options = b'\232\305\210\036 \n\036envoy.api.v2.core.BuildVersion'
  _globals['_EXTENSION'].fields_by_name['type_descriptor']._loaded_options = None
  _globals['_EXTENSION'].fields_by_name['type_descriptor']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_EXTENSION']._loaded_options = None
  _globals['_EXTENSION']._serialized_options = b'\232\305\210\036\035\n\033envoy.api.v2.core.Extension'
  _globals['_NODE_DYNAMICPARAMETERSENTRY']._loaded_options = None
  _globals['_NODE_DYNAMICPARAMETERSENTRY']._serialized_options = b'8\001'
  _globals['_NODE'].fields_by_name['listening_addresses']._loaded_options = None
  _globals['_NODE'].fields_by_name['listening_addresses']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_NODE']._loaded_options = None
  _globals['_NODE']._serialized_options = b'\232\305\210\036\030\n\026envoy.api.v2.core.Node'
  _globals['_METADATA_FILTERMETADATAENTRY']._loaded_options = None
  _globals['_METADATA_FILTERMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_METADATA_TYPEDFILTERMETADATAENTRY']._loaded_options = None
  _globals['_METADATA_TYPEDFILTERMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_METADATA'].fields_by_name['filter_metadata']._loaded_options = None
  _globals['_METADATA'].fields_by_name['filter_metadata']._serialized_options = b'\372B\t\232\001\006\"\004r\002\020\001'
  _globals['_METADATA'].fields_by_name['typed_filter_metadata']._loaded_options = None
  _globals['_METADATA'].fields_by_name['typed_filter_metadata']._serialized_options = b'\372B\t\232\001\006\"\004r\002\020\001'
  _globals['_METADATA']._loaded_options = None
  _globals['_METADATA']._serialized_options = b'\232\305\210\036\034\n\032envoy.api.v2.core.Metadata'
  _globals['_RUNTIMEUINT32']._loaded_options = None
  _globals['_RUNTIMEUINT32']._serialized_options = b'\232\305\210\036!\n\037envoy.api.v2.core.RuntimeUInt32'
  _globals['_RUNTIMEDOUBLE']._loaded_options = None
  _globals['_RUNTIMEDOUBLE']._serialized_options = b'\232\305\210\036!\n\037envoy.api.v2.core.RuntimeDouble'
  _globals['_RUNTIMEFEATUREFLAG'].fields_by_name['default_value']._loaded_options = None
  _globals['_RUNTIMEFEATUREFLAG'].fields_by_name['default_value']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_RUNTIMEFEATUREFLAG']._loaded_options = None
  _globals['_RUNTIMEFEATUREFLAG']._serialized_options = b'\232\305\210\036&\n$envoy.api.v2.core.RuntimeFeatureFlag'
  _globals['_KEYVALUE'].fields_by_name['key']._loaded_options = None
  _globals['_KEYVALUE'].fields_by_name['key']._serialized_options = b'\030\001\372B\010r\006\020\001(\200\200\001\222\307\206\330\004\0033.0'
  _globals['_KEYVALUE'].fields_by_name['value']._loaded_options = None
  _globals['_KEYVALUE'].fields_by_name['value']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_KEYVALUEPAIR'].fields_by_name['key']._loaded_options = None
  _globals['_KEYVALUEPAIR'].fields_by_name['key']._serialized_options = b'\372B\010r\006\020\001(\200\200\001'
  _globals['_KEYVALUEAPPEND'].fields_by_name['entry']._loaded_options = None
  _globals['_KEYVALUEAPPEND'].fields_by_name['entry']._serialized_options = b'\030\001\372B\005\212\001\002\010\001\222\307\206\330\004\0033.0'
  _globals['_KEYVALUEAPPEND'].fields_by_name['action']._loaded_options = None
  _globals['_KEYVALUEAPPEND'].fields_by_name['action']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_KEYVALUEMUTATION'].fields_by_name['remove']._loaded_options = None
  _globals['_KEYVALUEMUTATION'].fields_by_name['remove']._serialized_options = b'\372B\006r\004(\200\200\001'
  _globals['_QUERYPARAMETER'].fields_by_name['key']._loaded_options = None
  _globals['_QUERYPARAMETER'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_HEADERVALUE'].fields_by_name['key']._loaded_options = None
  _globals['_HEADERVALUE'].fields_by_name['key']._serialized_options = b'\372B\016r\014\020\001(\200\200\001\300\001\001\310\001\000'
  _globals['_HEADERVALUE'].fields_by_name['value']._loaded_options = None
  _globals['_HEADERVALUE'].fields_by_name['value']._serialized_options = b'\372B\014r\n(\200\200\001\300\001\002\310\001\000\362\230\376\217\005\014\022\nvalue_type'
  _globals['_HEADERVALUE'].fields_by_name['raw_value']._loaded_options = None
  _globals['_HEADERVALUE'].fields_by_name['raw_value']._serialized_options = b'\372B\010z\006\020\000\030\200\200\001\362\230\376\217\005\014\022\nvalue_type'
  _globals['_HEADERVALUE']._loaded_options = None
  _globals['_HEADERVALUE']._serialized_options = b'\232\305\210\036\037\n\035envoy.api.v2.core.HeaderValue'
  _globals['_HEADERVALUEOPTION'].fields_by_name['header']._loaded_options = None
  _globals['_HEADERVALUEOPTION'].fields_by_name['header']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_HEADERVALUEOPTION'].fields_by_name['append']._loaded_options = None
  _globals['_HEADERVALUEOPTION'].fields_by_name['append']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_HEADERVALUEOPTION'].fields_by_name['append_action']._loaded_options = None
  _globals['_HEADERVALUEOPTION'].fields_by_name['append_action']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_HEADERVALUEOPTION']._loaded_options = None
  _globals['_HEADERVALUEOPTION']._serialized_options = b'\232\305\210\036%\n#envoy.api.v2.core.HeaderValueOption'
  _globals['_HEADERMAP']._loaded_options = None
  _globals['_HEADERMAP']._serialized_options = b'\232\305\210\036\035\n\033envoy.api.v2.core.HeaderMap'
  _globals['_WATCHEDDIRECTORY'].fields_by_name['path']._loaded_options = None
  _globals['_WATCHEDDIRECTORY'].fields_by_name['path']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DATASOURCE'].oneofs_by_name['specifier']._loaded_options = None
  _globals['_DATASOURCE'].oneofs_by_name['specifier']._serialized_options = b'\370B\001'
  _globals['_DATASOURCE'].fields_by_name['filename']._loaded_options = None
  _globals['_DATASOURCE'].fields_by_name['filename']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DATASOURCE'].fields_by_name['environment_variable']._loaded_options = None
  _globals['_DATASOURCE'].fields_by_name['environment_variable']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DATASOURCE']._loaded_options = None
  _globals['_DATASOURCE']._serialized_options = b'\232\305\210\036\036\n\034envoy.api.v2.core.DataSource'
  _globals['_RETRYPOLICY_RETRYPRIORITY'].fields_by_name['name']._loaded_options = None
  _globals['_RETRYPOLICY_RETRYPRIORITY'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_RETRYPOLICY_RETRYHOSTPREDICATE'].fields_by_name['name']._loaded_options = None
  _globals['_RETRYPOLICY_RETRYHOSTPREDICATE'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_RETRYPOLICY'].fields_by_name['num_retries']._loaded_options = None
  _globals['_RETRYPOLICY'].fields_by_name['num_retries']._serialized_options = b'\362\230\376\217\005\r\n\013max_retries'
  _globals['_RETRYPOLICY']._loaded_options = None
  _globals['_RETRYPOLICY']._serialized_options = b'\232\305\210\036\037\n\035envoy.api.v2.core.RetryPolicy'
  _globals['_REMOTEDATASOURCE'].fields_by_name['http_uri']._loaded_options = None
  _globals['_REMOTEDATASOURCE'].fields_by_name['http_uri']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_REMOTEDATASOURCE'].fields_by_name['sha256']._loaded_options = None
  _globals['_REMOTEDATASOURCE'].fields_by_name['sha256']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REMOTEDATASOURCE']._loaded_options = None
  _globals['_REMOTEDATASOURCE']._serialized_options = b'\232\305\210\036$\n\"envoy.api.v2.core.RemoteDataSource'
  _globals['_ASYNCDATASOURCE'].oneofs_by_name['specifier']._loaded_options = None
  _globals['_ASYNCDATASOURCE'].oneofs_by_name['specifier']._serialized_options = b'\370B\001'
  _globals['_ASYNCDATASOURCE']._loaded_options = None
  _globals['_ASYNCDATASOURCE']._serialized_options = b'\232\305\210\036#\n!envoy.api.v2.core.AsyncDataSource'
  _globals['_TRANSPORTSOCKET'].fields_by_name['name']._loaded_options = None
  _globals['_TRANSPORTSOCKET'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TRANSPORTSOCKET']._loaded_options = None
  _globals['_TRANSPORTSOCKET']._serialized_options = b'\232\305\210\036#\n!envoy.api.v2.core.TransportSocket'
  _globals['_RUNTIMEFRACTIONALPERCENT'].fields_by_name['default_value']._loaded_options = None
  _globals['_RUNTIMEFRACTIONALPERCENT'].fields_by_name['default_value']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_RUNTIMEFRACTIONALPERCENT']._loaded_options = None
  _globals['_RUNTIMEFRACTIONALPERCENT']._serialized_options = b'\232\305\210\036,\n*envoy.api.v2.core.RuntimeFractionalPercent'
  _globals['_CONTROLPLANE']._loaded_options = None
  _globals['_CONTROLPLANE']._serialized_options = b'\232\305\210\036 \n\036envoy.api.v2.core.ControlPlane'
  _globals['_ROUTINGPRIORITY']._serialized_start=5557
  _globals['_ROUTINGPRIORITY']._serialized_end=5597
  _globals['_REQUESTMETHOD']._serialized_start=5600
  _globals['_REQUESTMETHOD']._serialized_end=5737
  _globals['_TRAFFICDIRECTION']._serialized_start=5739
  _globals['_TRAFFICDIRECTION']._serialized_end=5801
  _globals['_LOCALITY']._serialized_start=516
  _globals['_LOCALITY']._serialized_end=609
  _globals['_BUILDVERSION']._serialized_start=612
  _globals['_BUILDVERSION']._serialized_end=757
  _globals['_EXTENSION']._serialized_start=760
  _globals['_EXTENSION']._serialized_end=967
  _globals['_NODE']._serialized_start=970
  _globals['_NODE']._serialized_end=1620
  _globals['_NODE_DYNAMICPARAMETERSENTRY']._serialized_start=1457
  _globals['_NODE_DYNAMICPARAMETERSENTRY']._serialized_end=1541
  _globals['_METADATA']._serialized_start=1623
  _globals['_METADATA']._serialized_end=2023
  _globals['_METADATA_FILTERMETADATAENTRY']._serialized_start=1828
  _globals['_METADATA_FILTERMETADATAENTRY']._serialized_end=1906
  _globals['_METADATA_TYPEDFILTERMETADATAENTRY']._serialized_start=1908
  _globals['_METADATA_TYPEDFILTERMETADATAENTRY']._serialized_end=1988
  _globals['_RUNTIMEUINT32']._serialized_start=2025
  _globals['_RUNTIMEUINT32']._serialized_end=2124
  _globals['_RUNTIMEPERCENT']._serialized_start=2126
  _globals['_RUNTIMEPERCENT']._serialized_end=2210
  _globals['_RUNTIMEDOUBLE']._serialized_start=2212
  _globals['_RUNTIMEDOUBLE']._serialized_end=2311
  _globals['_RUNTIMEFEATUREFLAG']._serialized_start=2314
  _globals['_RUNTIMEFEATUREFLAG']._serialized_end=2461
  _globals['_KEYVALUE']._serialized_start=2463
  _globals['_KEYVALUE']._serialized_end=2538
  _globals['_KEYVALUEPAIR']._serialized_start=2540
  _globals['_KEYVALUEPAIR']._serialized_end=2619
  _globals['_KEYVALUEAPPEND']._serialized_start=2622
  _globals['_KEYVALUEAPPEND']._serialized_end=2972
  _globals['_KEYVALUEAPPEND_KEYVALUEAPPENDACTION']._serialized_start=2845
  _globals['_KEYVALUEAPPEND_KEYVALUEAPPENDACTION']._serialized_end=2972
  _globals['_KEYVALUEMUTATION']._serialized_start=2974
  _globals['_KEYVALUEMUTATION']._serialized_end=3073
  _globals['_QUERYPARAMETER']._serialized_start=3075
  _globals['_QUERYPARAMETER']._serialized_end=3128
  _globals['_HEADERVALUE']._serialized_start=3131
  _globals['_HEADERVALUE']._serialized_end=3314
  _globals['_HEADERVALUEOPTION']._serialized_start=3317
  _globals['_HEADERVALUEOPTION']._serialized_end=3744
  _globals['_HEADERVALUEOPTION_HEADERAPPENDACTION']._serialized_start=3575
  _globals['_HEADERVALUEOPTION_HEADERAPPENDACTION']._serialized_end=3700
  _globals['_HEADERMAP']._serialized_start=3746
  _globals['_HEADERMAP']._serialized_end=3845
  _globals['_WATCHEDDIRECTORY']._serialized_start=3847
  _globals['_WATCHEDDIRECTORY']._serialized_end=3888
  _globals['_DATASOURCE']._serialized_start=3891
  _globals['_DATASOURCE']._serialized_end=4144
  _globals['_RETRYPOLICY']._serialized_start=4147
  _globals['_RETRYPOLICY']._serialized_end=4758
  _globals['_RETRYPOLICY_RETRYPRIORITY']._serialized_start=4515
  _globals['_RETRYPOLICY_RETRYPRIORITY']._serialized_end=4614
  _globals['_RETRYPOLICY_RETRYHOSTPREDICATE']._serialized_start=4616
  _globals['_RETRYPOLICY_RETRYHOSTPREDICATE']._serialized_end=4720
  _globals['_REMOTEDATASOURCE']._serialized_start=4761
  _globals['_REMOTEDATASOURCE']._serialized_end=4963
  _globals['_ASYNCDATASOURCE']._serialized_start=4966
  _globals['_ASYNCDATASOURCE']._serialized_end=5152
  _globals['_TRANSPORTSOCKET']._serialized_start=5155
  _globals['_TRANSPORTSOCKET']._serialized_end=5312
  _globals['_RUNTIMEFRACTIONALPERCENT']._serialized_start=5315
  _globals['_RUNTIMEFRACTIONALPERCENT']._serialized_end=5480
  _globals['_CONTROLPLANE']._serialized_start=5482
  _globals['_CONTROLPLANE']._serialized_end=5555
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# Copyright 2022 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# A script to fetch total cpu seconds and memory data from prometheus.
# example usage: python3 prometheus.py
# --url=http://prometheus.prometheus.svc.cluster.local:9090
# --pod_type=driver --pod_type=clients --container_name=main
# --container_name=sidecar
"""Perform Prometheus range queries to obtain cpu and memory data.

This module performs range queries through Prometheus API to obtain
total cpu seconds and memory during a test run for given container
in given pods. The cpu data obtained is total cpu second used within
given period of time. The memory data was instant memory usage at
the query time.
"""
import argparse
import json
import logging
import statistics
import time
from typing import Any, Dict, List

from dateutil import parser
import requests


class Prometheus:
    """Objects which holds the start time, end time and query URL."""

    def __init__(
        self,
        url: str,
        start: str,
        end: str,
    ):
        self.url = url
        self.start = start
        self.end = end

    def _fetch_by_query(self, query: str) -> Dict[str, Any]:
        """Fetches the given query with time range.

        Fetch the given query within a time range. The pulling
        interval is every 5s, the actual data from the query is
        a time series.
        """
        resp = requests.get(
            self.url + "/api/v1/query_range",
            {"query": query, "start": self.start, "end": self.end, "step": 5},
        )
        resp.raise_for_status()
        return resp.json()

    def _fetch_cpu_for_pod(
        self, container_matcher: str, pod_name: str
    ) -> Dict[str, List[float]]:
        """Fetches the cpu data for each pod.

        Fetch total cpu seconds during the time range specified in the Prometheus instance
        for a pod. After obtain the cpu seconds, the data are trimmed from time series to
        a data list and saved in the Dict that keyed by the container names.

        Args:
            container_matcher:  A string consist one or more container name separated by |.
        """
        query = (
            'container_cpu_usage_seconds_total{job="kubernetes-cadvisor",pod="'
            + pod_name
            + '",container='
            + container_matcher
            + "}"
        )
        logging.debug("running prometheus query for cpu: %s", query)
        cpu_data = self._fetch_by_query(query)
        logging.debug("raw cpu data: %s", str(cpu_data))
        cpu_container_name_to_data_list = get_data_list_from_timeseries(
            cpu_data
        )
        return cpu_container_name_to_data_list

    def _fetch_memory_for_pod(
        self, container_matcher: str, pod_name: str
    ) -> Dict[str, List[float]]:
        """Fetches memory data for each pod.

        Fetch total memory data during the time range specified in the Prometheus instance
        for a pod. After obtain the memory data, the data are trimmed from time series to
        a data list and saved in the Dict that keyed by the container names.

        Args:
            container_matcher:  A string consist one or more container name separated by |.
        """
        query = (
            'container_memory_usage_bytes{job="kubernetes-cadvisor",pod="'
            + pod_name
            + '",container='
            + container_matcher
            + "}"
        )

        logging.debug("running prometheus query for memory: %s", query)
        memory_data = self._fetch_by_query(query)

        logging.debug("raw memory data: %s", str(memory_data))
        memory_container_name_to_data_list = get_data_list_from_timeseries(
            memory_data
        )

        return memory_container_name_to_data_list

    def fetch_cpu_and_memory_data(
        self, container_list: List[str], pod_dict: Dict[str, List[str]]
    ) -> Dict[str, Any]:
        """Fetch total cpu seconds and memory data for multiple pods.

        Args:
            container_list: A list of container names to fetch the data for.
            pod_dict: the pods to fetch data for, the pod_dict is keyed by
                      role of the pod: clients, driver and servers. The values
                      for the pod_dict are the list of pod names that consist
                      the same role specified in the key.
        """
        container_matcher = construct_container_matcher(container_list)
        processed_data = {}
        for role, pod_names in pod_dict.items():
            pod_data = {}
            for pod in pod_names:
                container_data = {}
                for container, data in self._fetch_cpu_for_pod(
                    container_matcher, pod
                ).items():
                    container_data[container] = {}
                    container_data[container]["cpuSeconds"] = (
                        compute_total_cpu_seconds(data)
                    )

                for container, data in self._fetch_memory_for_pod(
                    container_matcher, pod
                ).items():
                    container_data[container]["memoryMean"] = (
                        compute_average_memory_usage(data)
                    )

                pod_data[pod] = container_data
            processed_data[role] = pod_data
        return processed_data


def construct_container_matcher(container_list: List[str]) -> str:
    """Constructs the container matching string used in the
    prometheus query."""
    if len(container_list) == 0:
        raise Exception("no container name provided")

    containers_to_fetch = '"'
    if len(container_list) == 1:
        containers_to_fetch = container_list[0]
    else:
        containers_to_fetch = '~"' + container_list[0]
        for container in container_list[1:]:
            containers_to_fetch = containers_to_fetch + "|" + container
        containers_to_fetch = containers_to_fetch + '"'
    return containers_to_fetch


def get_data_list_from_timeseries(data: Any) -> Dict[str, List[float]]:
    """Constructs a Dict as keys are the container names and
    values are a list of data taken from given timeseries data."""
    if data["status"] != "success":
        raise Exception("command failed: " + data["status"] + str(data))
    if data["data"]["resultType"] != "matrix":
        raise Exception(
            "resultType is not matrix: " + data["data"]["resultType"]
        )

    container_name_to_data_list = {}
    for res in data["data"]["result"]:
        container_name = res["metric"]["container"]
        container_data_timeseries = res["values"]

        container_data = []
        for d in container_data_timeseries:
            container_data.append(float(d[1]))
        container_name_to_data_list[container_name] = container_data
    return container_name_to_data_list


def compute_total_cpu_seconds(cpu_data_list: List[float]) -> float:
    """Computes the total cpu seconds by CPUs[end]-CPUs[start]."""
    return cpu_data_list[len(cpu_data_list) - 1] - cpu_data_list[0]


def compute_average_memory_usage(memory_data_list: List[float]) -> float:
    """Computes the mean and for a given list of data."""
    return statistics.mean(memory_data_list)


def construct_pod_dict(
    node_info_file: str, pod_types: List[str]
) -> Dict[str, List[str]]:
    """Constructs a dict of pod names to be queried.

    Args:
        node_info_file: The file path contains the pod names to query.
            The pods' names are put into a Dict of list that keyed by the
            role name: clients, servers and driver.
    """
    with open(node_info_file, "r") as f:
        pod_names = json.load(f)
        pod_type_to_name = {"clients": [], "driver": [], "servers": []}

        for client in pod_names["Clients"]:
            pod_type_to_name["clients"].append(client["Name"])
        for server in pod_names["Servers"]:
            pod_type_to_name["servers"].append(server["Name"])

        pod_type_to_name["driver"].append(pod_names["Driver"]["Name"])

    pod_names_to_query = {}
    for pod_type in pod_types:
        pod_names_to_query[pod_type] = pod_type_to_name[pod_type]
    return pod_names_to_query


def convert_UTC_to_epoch(utc_timestamp: str) -> str:
    """Converts a utc timestamp string to epoch time string."""
    parsed_time = parser.parse(utc_timestamp)
    epoch = parsed_time.strftime("%s")
    return epoch


def main() -> None:
    argp = argparse.ArgumentParser(
        description="Fetch cpu and memory stats from prometheus"
    )
    argp.add_argument("--url", help="Prometheus base url", required=True)
    argp.add_argument(
        "--scenario_result_file",
        default="scenario_result.json",
        type=str,
        help="File contains epoch seconds for start and end time",
    )
    argp.add_argument(
        "--node_info_file",
        default="/var/data/qps_workers/node_info.json",
        help="File contains pod name to query the metrics for",
    )
    argp.add_argument(
        "--pod_type",
        action="append",
        help=(
            "Pod type to query the metrics for, the options are driver, client"
            " and server"
        ),
        choices=["driver", "clients", "servers"],
        required=True,
    )
    argp.add_argument(
        "--container_name",
        action="append",
        help="The container names to query the metrics for",
        required=True,
    )
    argp.add_argument(
        "--export_file_name",
        default="prometheus_query_result.json",
        type=str,
        help="Name of exported JSON file.",
    )
    argp.add_argument(
        "--quiet",
        default=False,
        help="Suppress informative output",
    )
    argp.add_argument(
        "--delay_seconds",
        default=0,
        type=int,
        help=(
            "Configure delay in seconds to perform Prometheus queries, default"
            " is 0"
        ),
    )
    args = argp.parse_args()

    if not args.quiet:
        logging.getLogger().setLevel(logging.DEBUG)

    with open(args.scenario_result_file, "r") as q:
        scenario_result = json.load(q)
        start_time = convert_UTC_to_epoch(
            scenario_result["summary"]["startTime"]
        )
        end_time = convert_UTC_to_epoch(scenario_result["summary"]["endTime"])
        p = Prometheus(
            url=args.url,
            start=start_time,
            end=end_time,
        )

    time.sleep(args.delay_seconds)

    pod_dict = construct_pod_dict(args.node_info_file, args.pod_type)
    processed_data = p.fetch_cpu_and_memory_data(
        container_list=args.container_name, pod_dict=pod_dict
    )
    processed_data["testDurationSeconds"] = float(end_time) - float(start_time)

    logging.debug(
        "%s: %s",
        args.export_file_name,
        json.dumps(processed_data, sort_keys=True, indent=4),
    )

    with open(args.export_file_name, "w", encoding="utf8") as export_file:
        json.dump(processed_data, export_file, sort_keys=True, indent=4)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/conformance/proto2/test_all_types_extensions.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/conformance/proto2/test_all_types_extensions.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr.conformance.proto2 import test_all_types_pb2 as cel_dot_expr_dot_conformance_dot_proto2_dot_test__all__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;cel/expr/conformance/proto2/test_all_types_extensions.proto\x12\x1b\x63\x65l.expr.conformance.proto2\x1a\x30\x63\x65l/expr/conformance/proto2/test_all_types.proto\"\xda\x03\n\x1cProto2ExtensionScopedMessage2=\n\tint64_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xed\x07 \x01(\x03\x32x\n\x19message_scoped_nested_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xee\x07 \x01(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes2y\n\x0fnested_enum_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xef\x07 \x01(\x0e\x32\x34.cel.expr.conformance.proto2.TestAllTypes.NestedEnum2\x85\x01\n&message_scoped_repeated_test_all_types\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xf0\x07 \x03(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes:=\n\tint32_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xe8\x07 \x01(\x05:i\n\nnested_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xe9\x07 \x01(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes:q\n\x12test_all_types_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xea\x07 \x01(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes:y\n\x0fnested_enum_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xeb\x07 \x01(\x0e\x32\x34.cel.expr.conformance.proto2.TestAllTypes.NestedEnum:v\n\x17repeated_test_all_types\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xec\x07 \x03(\x0b\x32).cel.expr.conformance.proto2.TestAllTypesB_\n\x1f\x64\x65v.cel.expr.conformance.proto2B\x16TestAllTypesExtensionsP\x01Z\x1f\x63\x65l.dev/expr/conformance/proto2\xf8\x01\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.conformance.proto2.test_all_types_extensions_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\037dev.cel.expr.conformance.proto2B\026TestAllTypesExtensionsP\001Z\037cel.dev/expr/conformance/proto2\370\001\001'
  _globals['_PROTO2EXTENSIONSCOPEDMESSAGE']._serialized_start=143
  _globals['_PROTO2EXTENSIONSCOPEDMESSAGE']._serialized_end=617
# @@protoc_insertion_point(module_scope)



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Setup module for admin interface in gRPC Python."""

import os
import sys

import setuptools

# Manually insert the source directory into the Python path for local module
# imports to succeed
sys.path.insert(0, os.path.abspath("."))

import grpc_version
import python_version

INSTALL_REQUIRES = (
    "grpcio-channelz>={version}".format(version=grpc_version.VERSION),
    "grpcio-csds>={version}".format(version=grpc_version.VERSION),
)
SETUP_REQUIRES = INSTALL_REQUIRES

PYTHON_REQUIRES = f">={python_version.MIN_PYTHON_VERSION}"

if __name__ == "__main__":
    setuptools.setup(
        python_requires=f">={python_version.MIN_PYTHON_VERSION}",
        install_requires=INSTALL_REQUIRES,
        setup_requires=SETUP_REQUIRES,
    )



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

cimport cpython
from cython.operator cimport dereference

import enum
import functools
import logging
import os
from threading import Thread
from typing import AnyStr, Dict, List, Mapping, Tuple, Union

from grpc_observability import _observability

# Time we wait for batch exporting census data
# TODO(xuanwn): change interval to a more appropriate number
CENSUS_EXPORT_BATCH_INTERVAL_SECS = float(os.environ.get('GRPC_PYTHON_CENSUS_EXPORT_BATCH_INTERVAL_SECS', 0.5))
GRPC_PYTHON_CENSUS_EXPORT_THREAD_TIMEOUT = float(os.environ.get('GRPC_PYTHON_CENSUS_EXPORT_THREAD_TIMEOUT', 10))
cdef const char* CLIENT_CALL_TRACER = "client_call_tracer"
cdef const char* SERVER_CALL_TRACER_FACTORY = "server_call_tracer_factory"
cdef bint GLOBAL_SHUTDOWN_EXPORT_THREAD = False
cdef object GLOBAL_EXPORT_THREAD

PLUGIN_IDENTIFIER_SEP = ","

_LOGGER = logging.getLogger(__name__)


class _CyMetricsName:
  CY_CLIENT_API_LATENCY = kRpcClientApiLatencyMeasureName
  CY_CLIENT_SNET_MESSSAGES_PER_RPC = kRpcClientSentMessagesPerRpcMeasureName
  CY_CLIENT_SEND_BYTES_PER_RPC = kRpcClientSentBytesPerRpcMeasureName
  CY_CLIENT_RECEIVED_MESSAGES_PER_RPC = kRpcClientReceivedMessagesPerRpcMeasureName
  CY_CLIENT_RECEIVED_BYTES_PER_RPC = kRpcClientReceivedBytesPerRpcMeasureName
  CY_CLIENT_ROUNDTRIP_LATENCY = kRpcClientRoundtripLatencyMeasureName
  CY_CLIENT_COMPLETED_RPC = kRpcClientCompletedRpcMeasureName
  CY_CLIENT_SERVER_LATENCY = kRpcClientServerLatencyMeasureName
  CY_CLIENT_STARTED_RPCS = kRpcClientStartedRpcsMeasureName
  CY_CLIENT_RETRIES_PER_CALL = kRpcClientRetriesPerCallMeasureName
  CY_CLIENT_TRANSPARENT_RETRIES_PER_CALL = kRpcClientTransparentRetriesPerCallMeasureName
  CY_CLIENT_RETRY_DELAY_PER_CALL = kRpcClientRetryDelayPerCallMeasureName
  CY_CLIENT_TRANSPORT_LATENCY = kRpcClientTransportLatencyMeasureName
  CY_SERVER_SENT_MESSAGES_PER_RPC = kRpcServerSentMessagesPerRpcMeasureName
  CY_SERVER_SENT_BYTES_PER_RPC = kRpcServerSentBytesPerRpcMeasureName
  CY_SERVER_RECEIVED_MESSAGES_PER_RPC = kRpcServerReceivedMessagesPerRpcMeasureName
  CY_SERVER_RECEIVED_BYTES_PER_RPC = kRpcServerReceivedBytesPerRpcMeasureName
  CY_SERVER_SERVER_LATENCY = kRpcServerServerLatencyMeasureName
  CY_SERVER_COMPLETED_RPC = kRpcServerCompletedRpcMeasureName
  CY_SERVER_STARTED_RPCS = kRpcServerStartedRpcsMeasureName

@enum.unique
class MetricsName(enum.Enum):
  CLIENT_STARTED_RPCS = _CyMetricsName.CY_CLIENT_STARTED_RPCS
  CLIENT_API_LATENCY = _CyMetricsName.CY_CLIENT_API_LATENCY
  CLIENT_SNET_MESSSAGES_PER_RPC = _CyMetricsName.CY_CLIENT_SNET_MESSSAGES_PER_RPC
  CLIENT_SEND_BYTES_PER_RPC = _CyMetricsName.CY_CLIENT_SEND_BYTES_PER_RPC
  CLIENT_RECEIVED_MESSAGES_PER_RPC = _CyMetricsName.CY_CLIENT_RECEIVED_MESSAGES_PER_RPC
  CLIENT_RECEIVED_BYTES_PER_RPC = _CyMetricsName.CY_CLIENT_RECEIVED_BYTES_PER_RPC
  CLIENT_ROUNDTRIP_LATENCY = _CyMetricsName.CY_CLIENT_ROUNDTRIP_LATENCY
  CLIENT_COMPLETED_RPC = _CyMetricsName.CY_CLIENT_COMPLETED_RPC
  CLIENT_SERVER_LATENCY = _CyMetricsName.CY_CLIENT_SERVER_LATENCY
  CLIENT_RETRIES_PER_CALL = _CyMetricsName.CY_CLIENT_RETRIES_PER_CALL
  CLIENT_TRANSPARENT_RETRIES_PER_CALL = _CyMetricsName.CY_CLIENT_TRANSPARENT_RETRIES_PER_CALL
  CLIENT_RETRY_DELAY_PER_CALL = _CyMetricsName.CY_CLIENT_RETRY_DELAY_PER_CALL
  SERVER_SENT_MESSAGES_PER_RPC = _CyMetricsName.CY_SERVER_SENT_MESSAGES_PER_RPC
  SERVER_SENT_BYTES_PER_RPC = _CyMetricsName.CY_SERVER_SENT_BYTES_PER_RPC
  SERVER_RECEIVED_MESSAGES_PER_RPC = _CyMetricsName.CY_SERVER_RECEIVED_MESSAGES_PER_RPC
  SERVER_RECEIVED_BYTES_PER_RPC = _CyMetricsName.CY_SERVER_RECEIVED_BYTES_PER_RPC
  SERVER_SERVER_LATENCY = _CyMetricsName.CY_SERVER_SERVER_LATENCY
  SERVER_COMPLETED_RPC = _CyMetricsName.CY_SERVER_COMPLETED_RPC
  SERVER_STARTED_RPCS = _CyMetricsName.CY_SERVER_STARTED_RPCS

# Delay map creation due to circular dependencies
_CY_METRICS_NAME_TO_PY_METRICS_NAME_MAPPING = {x.value: x for x in MetricsName}

def cyobservability_init(object exporter) -> None:
  exporter: _observability.Exporter

  NativeObservabilityInit()
  _start_exporting_thread(exporter)


def _start_exporting_thread(object exporter) -> None:
  exporter: _observability.Exporter

  global GLOBAL_EXPORT_THREAD
  global GLOBAL_SHUTDOWN_EXPORT_THREAD
  GLOBAL_SHUTDOWN_EXPORT_THREAD = False
  # TODO(xuanwn): Change it to daemon thread.
  GLOBAL_EXPORT_THREAD = Thread(target=_export_census_data, args=(exporter,))
  GLOBAL_EXPORT_THREAD.start()

def activate_config(object py_config) -> None:
  py_config: "_observability_config.GcpObservabilityConfig"

  if (py_config.tracing_enabled):
    EnablePythonCensusTracing(True);
    # Save sampling rate to global sampler.
    ProbabilitySampler.Get().SetThreshold(py_config.sampling_rate)

  if (py_config.stats_enabled):
    EnablePythonCensusStats(True);

def activate_stats() -> None:
  EnablePythonCensusStats(True);

def create_client_call_tracer(bytes method_name, bytes target, bytes trace_id, str identifier,
                              dict exchange_labels, object enabled_optional_labels,
                              bint registered_method, bytes parent_span_id=b'') -> cpython.PyObject:
  """Create a ClientCallTracer and save to PyCapsule.

  Returns: A grpc_observability._observability.ClientCallTracerCapsule object.
  """
  cdef char* c_method = cpython.PyBytes_AsString(method_name)
  cdef char* c_target = cpython.PyBytes_AsString(target)
  cdef char* c_trace_id = cpython.PyBytes_AsString(trace_id)
  cdef char* c_parent_span_id = cpython.PyBytes_AsString(parent_span_id)
  identifier_bytes = _encode(identifier)
  cdef char* c_identifier = cpython.PyBytes_AsString(identifier_bytes)
  cdef vector[Label] c_labels = _labels_to_c_labels(exchange_labels)
  cdef bint add_csm_optional_labels = False

  for label_type in enabled_optional_labels:
    if label_type == _observability.OptionalLabelType.XDS_SERVICE_LABELS:
      add_csm_optional_labels = True

  cdef void* call_tracer = CreateClientCallTracer(c_method, c_target, c_trace_id, c_parent_span_id,
                                                  c_identifier, c_labels, add_csm_optional_labels,
                                                  registered_method)
  capsule = cpython.PyCapsule_New(call_tracer, CLIENT_CALL_TRACER, NULL)
  return capsule


def create_server_call_tracer_factory_capsule(dict exchange_labels, str identifier) -> cpython.PyObject:
  """Create a ServerCallTracerFactory and save to PyCapsule.

  Returns: A grpc_observability._observability.ServerCallTracerFactoryCapsule object.
  """
  cdef vector[Label] c_labels = _labels_to_c_labels(exchange_labels)
  cdef char* c_identifier = cpython.PyBytes_AsString(_encode(identifier))
  cdef void* call_tracer_factory = CreateServerCallTracerFactory(c_labels, c_identifier)
  capsule = cpython.PyCapsule_New(call_tracer_factory, SERVER_CALL_TRACER_FACTORY, NULL)
  return capsule


def _c_label_to_labels(vector[Label] c_labels) -> Dict[str, AnyStr]:
  py_labels = {}
  for label in c_labels:
    py_labels[_decode(label.key)] = label.value
  return py_labels


def _labels_to_c_labels(dict py_labels) -> vector[Label]:
  cdef vector[Label] c_labels
  cdef Label label

  for key, value in py_labels.items():
      label.key = _encode(key)
      label.value = _encode(value)
      c_labels.push_back(label)

  return c_labels


def _c_measurement_to_measurement(object measurement
  ) -> Mapping[str, Union[enum, Mapping[str, Union[float, int], bool]]]:
  """Convert Cython Measurement to Python measurement.

  Args:
  measurement: Actual measurement represented by Cython type Measurement, using object here
   since Cython refuse to automatically convert a union with unsafe type combinations.

  Returns:
   A mapping object with keys and values as following:
    name -> cMetricsName
    type -> MeasurementType
    registered_method -> bool
    include_exchange_labels -> bool
    value -> {value_double: float | value_int: int}
  """
  measurement: Measurement

  py_measurement = {}
  py_measurement['name'] = measurement['name']
  py_measurement['type'] = measurement['type']
  py_measurement['registered_method'] = measurement['registered_method']
  py_measurement['include_exchange_labels'] = measurement['include_exchange_labels']
  if measurement['type'] == kMeasurementDouble:
    py_measurement['value'] = {'value_double': measurement['value']['value_double']}
  else:
    py_measurement['value'] = {'value_int': measurement['value']['value_int']}
  return py_measurement


def _c_annotation_to_annotations(vector[Annotation] c_annotations) -> List[Tuple[str, str]]:
  py_annotations = []
  for annotation in c_annotations:
    py_annotations.append((_decode(annotation.time_stamp),
                          _decode(annotation.description)))
  return py_annotations


def observability_deinit() -> None:
  _shutdown_exporting_thread()
  EnablePythonCensusStats(False)
  EnablePythonCensusTracing(False)


@functools.lru_cache(maxsize=None)
def _cy_metric_name_to_py_metric_name(cMetricsName metric_name) -> MetricsName:
  try:
    return _CY_METRICS_NAME_TO_PY_METRICS_NAME_MAPPING[metric_name]
  except KeyError:
    raise ValueError('Invalid metric name %s' % metric_name)


def _get_stats_data(object measurement, object labels, object identifier) -> _observability.StatsData:
  """Convert a Python measurement to StatsData.

  Args:
  measurement: A dict of type Mapping[str, Union[enum, Mapping[str, Union[float, int]]]]
    with keys and values as following:
      name -> cMetricsName
      type -> MeasurementType
      registered_method -> bool
      include_exchange_labels -> bool
      value -> {value_double: float | value_int: int}
  labels: Labels assciociated with stats data with type of Mapping[str, AnyStr].
  identifier: Specifies the plugins associated with this stats data.
  """
  measurement: Measurement
  labels: Mapping[str, AnyStr]

  metric_name = _cy_metric_name_to_py_metric_name(measurement['name'])
  identifiers = set(identifier.split(PLUGIN_IDENTIFIER_SEP))
  if measurement['type'] == kMeasurementDouble:
    py_stat = _observability.StatsData(name=metric_name, measure_double=True,
                                       value_float=measurement['value']['value_double'],
                                       labels=labels,
                                       identifiers=identifiers,
                                       registered_method=measurement['registered_method'],
                                       include_exchange_labels=measurement['include_exchange_labels'],)
  else:
    py_stat = _observability.StatsData(name=metric_name, measure_double=False,
                                       value_int=measurement['value']['value_int'],
                                       labels=labels,
                                       identifiers=identifiers,
                                       registered_method=measurement['registered_method'],
                                       include_exchange_labels=measurement['include_exchange_labels'],)
  return py_stat


def _get_tracing_data(SpanCensusData span_data, vector[Label] span_labels,
                      vector[Annotation] span_annotations) -> _observability.TracingData:
  py_span_labels = _c_label_to_labels(span_labels)
  py_span_annotations = _c_annotation_to_annotations(span_annotations)
  return _observability.TracingData(name=_decode(span_data.name),
                                    start_time = _decode(span_data.start_time),
                                    end_time = _decode(span_data.end_time),
                                    trace_id = _decode(span_data.trace_id),
                                    span_id = _decode(span_data.span_id),
                                    parent_span_id = _decode(span_data.parent_span_id),
                                    status = _decode(span_data.status),
                                    should_sample = span_data.should_sample,
                                    child_span_count = span_data.child_span_count,
                                    span_labels = py_span_labels,
                                    span_annotations = py_span_annotations)


def _record_rpc_latency(object exporter, str method, str target, float rpc_latency,
                        str status_code, str identifier, bint registered_method) -> None:
  exporter: _observability.Exporter

  measurement = {}
  measurement['name'] = kRpcClientApiLatencyMeasureName
  measurement['type'] = kMeasurementDouble
  measurement['value'] = {'value_double': rpc_latency}
  measurement['registered_method'] = registered_method
  measurement['include_exchange_labels'] = False

  labels = {}
  labels[_decode(kClientMethod)] = method.strip("/")
  labels[_decode(kClientTarget)] = target
  labels[_decode(kClientStatus)] = status_code
  metric = _get_stats_data(measurement, labels, identifier)
  exporter.export_stats_data([metric])


cdef void _export_census_data(object exporter):
  """Main function running in export thread."""
  exporter: _observability.Exporter

  cdef int export_interval_ms = CENSUS_EXPORT_BATCH_INTERVAL_SECS * 1000
  while True:
    with nogil:
      while not GLOBAL_SHUTDOWN_EXPORT_THREAD:
        lk = new unique_lock[mutex](g_census_data_buffer_mutex)
        # Wait for next batch of census data OR timeout at fixed interval.
        # Batch export census data to minimize the time we acquire the GIL.
        AwaitNextBatchLocked(dereference(lk), export_interval_ms)

        # Break only when buffer have data
        if not g_census_data_buffer.empty():
          del lk
          break
        else:
          del lk

    _flush_census_data(exporter)

    if GLOBAL_SHUTDOWN_EXPORT_THREAD:
      break # Break to shutdown exporting thead

  # Flush one last time before shutdown thread
  _flush_census_data(exporter)


cdef void _flush_census_data(object exporter):
  exporter: _observability.Exporter

  lk = new unique_lock[mutex](g_census_data_buffer_mutex)
  if g_census_data_buffer.empty():
    del lk
    return
  py_metrics_batch = []
  py_spans_batch = []
  while not g_census_data_buffer.empty():
    c_census_data = g_census_data_buffer.front()
    if c_census_data.type == kMetricData:
      py_labels = _c_label_to_labels(c_census_data.labels)
      py_identifier = _decode(c_census_data.identifier)
      py_measurement = _c_measurement_to_measurement(c_census_data.measurement_data)
      py_metric = _get_stats_data(py_measurement, py_labels, py_identifier)
      py_metrics_batch.append(py_metric)
    else:
      py_span = _get_tracing_data(c_census_data.span_data, c_census_data.span_data.span_labels,
                                  c_census_data.span_data.span_annotations)
      py_spans_batch.append(py_span)
    g_census_data_buffer.pop()

  del lk
  exporter.export_stats_data(py_metrics_batch)
  exporter.export_tracing_data(py_spans_batch)


cdef void _shutdown_exporting_thread():
  with nogil:
    global GLOBAL_SHUTDOWN_EXPORT_THREAD
    GLOBAL_SHUTDOWN_EXPORT_THREAD = True
    g_census_data_buffer_cv.notify_all()
  GLOBAL_EXPORT_THREAD.join(timeout=GRPC_PYTHON_CENSUS_EXPORT_THREAD_TIMEOUT)


cdef str _decode(bytes bytestring):
  if isinstance(bytestring, (str,)):
    return <str>bytestring
  else:
    try:
      return bytestring.decode('utf8')
    except UnicodeDecodeError:
      _LOGGER.exception('Invalid encoding on %s', bytestring)
      return bytestring.decode('latin1')


cdef bytes _encode(object string_or_none):
  if string_or_none is None:
    return b''
  elif isinstance(string_or_none, (bytes,)):
    return <bytes>string_or_none
  elif isinstance(string_or_none, (unicode,)):
    return string_or_none.encode('utf8')
  else:
    raise TypeError('Expected str, not {}'.format(type(string_or_none)))
